//*****************************************************************************
//
//    IMPEX.DLL is a Windows-only module for importing (to PCM WAV) sound files
//    in other formats. It is based on excellent library LIBSNDFILE v 0.27
//    written by Erik de Castro Lopo
//    (http://www.zip.com.au/~erikd/libsndfile/, erikd@zip.com.au).
//
//    Note: obviously you'll need to compile LIBSNDFILE first before linking it
//    to build the dll. Once it is compiled just put in the project directory.
//
//    LIBSNDFILE is released under the terms of the GNU Lesser General Public
//    License (LGPL). Consequently, IMPEX.DLL is also released under the terms
//    of LGPL (see LGPL.txt for details).
//
//*****************************************************************************

#pragma link "libsndfile.lib"
#include <sndfile.h>
#include <condefs.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#define XPORT extern "C" __declspec(dllexport)
USERES("impex.res");

//---------------------------------------------------------------------------

HWND         caller ;
char*        in_file ;
char*        out_file ;
char*        status ;
bool         is_aborted ;
int          WM_IMPEXPROGRESSMSG = WM_USER + 1111 ;
int          WM_IMPEXERRORMSG    = WM_USER + 1112 ;
int          WM_IMPEXSTATUSMSG   = WM_USER + 1113 ;

struct       params
             {
             HWND  caller;
             char* in_file;
             char* out_file;
             int   out_format;
             };

struct       short_info
             {
             unsigned int	samplerate ;
             unsigned int	samples ;
             unsigned int	channels ;
             unsigned int	pcmbitwidth ;
             bool               sucess;
             } ;

short_info   file_info;

//---------------------------------------------------------------------------

XPORT short_info WINAPI GetFileInfo(HWND handle, char* filename)
{
  SNDFILE	*infile;
  SF_INFO	info ;

  caller = handle;

  infile = sf_open_read (filename, &info);
  if (! infile)
  {
     file_info.channels = 0;
     file_info.pcmbitwidth = 0;
     file_info.samplerate = 0;
     file_info.samples = 0;
     file_info.sucess = false;
     return file_info;

  } ;

  file_info.channels = info.channels;
  file_info.pcmbitwidth = info.pcmbitwidth ;
  file_info.samplerate = info.samplerate ;
  file_info.samples = info.samples ;
  file_info.sucess = true;
  sf_close (infile);
  return file_info;
}

//---------------------------------------------------------------------------

int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*){return 1;}

//---------------------------------------------------------------------------

XPORT int WINAPI SetParams(params arg)
{
  caller        = arg.caller ;
  in_file       = arg.in_file ;
  out_file      = arg.out_file ;
  return 0;
}

//---------------------------------------------------------------------------

XPORT char* WINAPI GetStatus(){return status;}

//---------------------------------------------------------------------------

XPORT int WINAPI AbortProcess()
{
   is_aborted = true;
   return 0 ;
}

//---------------------------------------------------------------------------

XPORT int WINAPI Import()
{
  SNDFILE	*infile, *outfile ;
  SF_INFO	info ;
  int		out_type ;
  int		out_subtype ;
  int           out_samplesize = 0;
  double	data_buffer [4096] ;
  unsigned int	actual, requested;
  unsigned int  data_read;
  int           file_size;

  infile = sf_open_read (in_file, &info);
  if (! infile)
  {
     status = "Unable to open input file" ;
     SendMessage(caller, WM_IMPEXERRORMSG, 0, 0);
     return -1 ;
  } ;

  file_size =  info.samples*info.channels;    // in samples, NOT bytes
  data_read = 0;
  info.format = info.format & SF_FORMAT_SUBMASK;
  info.format = SF_FORMAT_WAV | info.format;
  out_type = info.format & SF_FORMAT_TYPEMASK ;
  out_subtype = SF_FORMAT_PCM ;

  if (out_subtype) info.format = out_type | out_subtype ;
  else info.format = out_type | (info.format & SF_FORMAT_SUBMASK) ;

  if (out_samplesize) info.pcmbitwidth = out_samplesize ;

  outfile = sf_open_write (out_file, &info);
  if (! outfile)
   {
      status = "Unable to open output file" ;
      SendMessage(caller, WM_IMPEXERRORMSG, 0, 0);
      sf_close (infile) ;
      return  -1 ;
   } ;

  is_aborted = false ;
  status = "Processing";
  SendMessage(caller, WM_IMPEXSTATUSMSG, 0, 0);

  actual = requested = 4096 / info.channels;

  while (actual == requested)
   {
      if (is_aborted)
      {
        sf_close (infile) ;
        sf_close (outfile) ;
        status = "Cancelled";
        SendMessage(caller, WM_IMPEXSTATUSMSG, 0, 0);
        return 0;
      }

      actual = sf_read_double (infile, data_buffer, requested, 0) ;
      sf_write_double (outfile, data_buffer, actual, 0) ;
      data_read = data_read + actual;
      SendMessage(caller, WM_IMPEXPROGRESSMSG, 0, (int)(100 * data_read / file_size));
   } ;


  sf_close (infile) ;
  sf_close (outfile) ;
  status = "Finished";
  SendMessage(caller, WM_IMPEXSTATUSMSG, 0, 0);
  return 0 ;
}

